@@ -6,7 +6,6 @@
 import numpy as np
 import logging
 
-from collections import OrderedDict
 import importlib
 import glob
 import os
@@ -30,19 +29,18 @@ class Dispatcher():
     def __init__(self, config=None, section=None):
         self.log = logging.getLogger(self.__class__.__name__)
 
-        # Todo: This needs to be renamed to something more sensible.
         self.config = config
         self.section = section
 
         pathlist = []
-        pathlist.extend(config['general'].get('pathlist_objects', []))
+        pathlist.extend(config['general'].get('pathlist', []))
         pathlist.extend(config['general'].get('pathlist_default', []))
         self.pathlist = pathlist
         
-        self.objects = OrderedDict()
-        self.meta = OrderedDict()
-        self.image = OrderedDict()
-        self.history = OrderedDict()
+        self.objects = dict()
+        self.meta = dict()
+        self.image = dict()
+        self.history = dict()
 
     def instantiate(self, names=None):
         if names is None:
@@ -87,11 +85,11 @@ class Dispatcher():
             objectname = objectname[1:]
             name_list.append(objectname)
 
-        output = OrderedDict()
+        output = dict()
         for ii, ff in enumerate(name_list):
             output[ff] = {
-                'filepath': filepath_list[ii]
-                , 'name': name_list[ii]
+                'filepath': filepath_list[ii],
+                'name': name_list[ii]
             }
 
         return output
@@ -128,7 +126,7 @@ class Dispatcher():
             obj.check_param(*args, **kwargs)
 
     def get_config(self, *args, **kwargs):
-        config = OrderedDict()
+        config = dict()
         for key, obj in self.objects.items():
             config[key] = obj.get_config(*args, **kwargs)
         return config
@@ -157,7 +155,7 @@ class Dispatcher():
             rays = obj.generate_rays()
 
             if keep_meta:
-                self.meta[key] = OrderedDict()
+                self.meta[key] = dict()
                 self.meta[key]['num_out'] = np.sum(rays['mask'])
             
             if keep_history:
@@ -182,7 +180,7 @@ class Dispatcher():
             profiler.stop('Dispatcher: trace_global')
 
             if keep_meta:
-                self.meta[key] = OrderedDict()
+                self.meta[key] = dict()
                 self.meta[key]['num_out'] = np.sum(rays['mask'])
             
             if keep_history:
