@@ -14,7 +14,7 @@ from xicsrt.util import profiler
 from xicsrt.tools.xicsrt_doc import dochelper
 from xicsrt.objects._GeometryObject import GeometryObject
 
-from xicsrt.tools import aperture
+from xicsrt.tools import xicsrt_aperture
 
 @dochelper
 class XicsrtOpticGeneric(GeometryObject):
@@ -44,6 +44,13 @@ class XicsrtOpticGeneric(GeometryObject):
           The pixel size, used for binning rays into images.
           This is currently a single number signifying square pixels.
 
+        aperture: dict or array (None)
+          Define one or more apertures to to apply to this optic.
+          Each aperture is defined as a dictionary with the following keys:
+          shape, size, origin, logic. The origin and logic field keys are
+          optional. The interpretation of size will depend on the provided
+          shape.
+
         trace_local: bool (False)
           If true: transform rays to optic local coordinates before raytracing,
           do raytracing in local coordinates, then transform back to global
@@ -56,17 +63,10 @@ class XicsrtOpticGeneric(GeometryObject):
 
         check_size: bool (true)
           Perform a check for whether the rays intersect the optic within the
-          defined bounds (usually defined by 'xsize' an 'ysize'). If set to
+          defined bounds (usually defined by 'xsize' and 'ysize'). If set to
           `False` all rays with a defined reflection/transmission condition
           will be traced if an intersection can be determined.
 
-        aperture: dict or array (None)
-          Define one or more apertures to to apply to this optic.
-          Each aperture is defined as a dictionary with the following keys:
-          shape, size, origin, logic. The origin and logic field keys are
-          optional. The interpretation of size will depend on the provided
-          shape.
-
         check_aperture: bool (true)
           Perform a check for whether the rays intersect the optic within the
           defined bounds (usually defined by 'xsize' an 'ysize'). If set to
@@ -269,12 +269,15 @@ class XicsrtOpticGeneric(GeometryObject):
         m = rays['mask']
 
         if self.param['check_aperture']:
-            m_aperture = aperture.aperture_mask(X_local, m, self.param['aperture'])
+            m_aperture = xicsrt_aperture.aperture_mask(X_local, m, self.param['aperture'])
             m[m] = m_aperture[m]
         
         return rays
 
     def generate_normals(self, rays, X):
+        """
+        The Generic optic is flat, so the normal direction is always the zaxis.
+        """
         m = rays['mask']
         normals = np.zeros(X.shape, dtype=np.float64)
         normals[m] = self.param['zaxis']
