@@ -10,17 +10,15 @@ Contains the main functions that are called to perform raytracing.
 """
 
 import numpy as np
-from PIL import Image
 import logging
 import os
 
 from copy import deepcopy
-from collections import OrderedDict
 
 from xicsrt.util import profiler
 
 from xicsrt import xicsrt_config
-from xicsrt import xicsrt_input
+from xicsrt import xicsrt_io
 from xicsrt.objects._Dispatcher import Dispatcher
 from xicsrt.objects._RayArray import RayArray
 
@@ -62,7 +60,7 @@ def raytrace(config):
             random_seed += ii
         config_run['general']['random_seed'] = random_seed
         
-        iteration = raytrace_single(config_run, internal=True)
+        iteration = raytrace_single(config_run, _internal=True)
         output_list.append(iteration)
         
     output = combine_raytrace(output_list)
@@ -71,17 +69,20 @@ def raytrace(config):
     output['config']['general']['output_run_suffix'] = config['general']['output_run_suffix']
     output['config']['general']['random_seed'] = config['general']['random_seed']
 
+    if config['general']['save_config']:
+        xicsrt_io.save_config(output['config'])
     if config['general']['save_images']:
-        save_images(output)
+        xicsrt_io.save_images(output)
+    if config['general']['save_results']:
+        xicsrt_io.save_results(output)
     if config['general']['print_results']:
         print_raytrace(output)
-    if config['general']['save_config']:
-        xicsrt_input.save_config(output['config'])
-        
+
     profiler.stop('raytrace_multi')
     return output
 
-def raytrace_single(config, internal=False):
+
+def raytrace_single(config, _internal=False):
     """
     Perform a single raytrace run consisting of multiple iterations.
 
@@ -89,12 +90,19 @@ def raytrace_single(config, internal=False):
     those that are lost (found and lost). The found ray history will be
     returned in full. The lost ray history will be truncated to allow
     analysis of lost ray pattern while still limiting memory usage.
+
+    private keywords
+    ================
+    _internal : bool (False)
+      Used when calling this function from `raytrace` as part of the execution
+      of multiple runs. Controls how `history_max_lost` is handled along with
+      how `save_config` and `save_results` are interpreted.
     """
     profiler.start('raytrace')
 
     # Update the default config with the user config.
+    config = xicsrt_io.config_to_numpy(config)
     config = xicsrt_config.get_config(config)
-    config = xicsrt_config.config_to_numpy(config)
     check_config(config)
 
     logging.info('Seeding np.random with {}'.format(config['general']['random_seed']))
@@ -103,6 +111,9 @@ def raytrace_single(config, internal=False):
     num_iter = config['general']['number_of_iter']
     max_lost_iter = int(config['general']['history_max_lost']/num_iter)
 
+    if _internal:
+        max_lost_iter = max_lost_iter//config['general']['number_of_runs']
+
     # Setup the dispatchers.
     if 'filters' in config:
         logging.debug("Creating filters")
@@ -140,19 +151,22 @@ def raytrace_single(config, internal=False):
 
     output = combine_raytrace(output_list)
 
-    if internal is False:
+    if _internal is False:
         if config['general']['print_results']:
             print_raytrace(output)
         if config['general']['save_config']:
-            xicsrt_input.save_config(output['config'])
+            xicsrt_io.save_config(output['config'])
+        if config['general']['save_results']:
+            xicsrt_io.save_results(output)
 
     if config['general']['save_images']:
-        save_images(output)
+        xicsrt_io.save_images(output)
 
     profiler.stop('raytrace')
     # profiler.report()
     return output
 
+
 def _raytrace_iter(config, sources, optics):
     """ 
     Perform a single iteration of raytracing with the given sources and optics.
@@ -172,9 +186,9 @@ def _raytrace_iter(config, sources, optics):
     rays = optics.trace(rays, keep_history=keep_history, keep_images=keep_images)
 
     # Combine sources and optics outputs.
-    meta    = OrderedDict()
-    image   = OrderedDict()
-    history = OrderedDict()
+    meta    = dict()
+    image   = dict()
+    history = dict()
     
     if keep_meta:
         for key in sources.meta:
@@ -194,7 +208,7 @@ def _raytrace_iter(config, sources, optics):
         for key in optics.history:
             history[key] = optics.history[key]
 
-    output = OrderedDict()
+    output = dict()
     output['config'] = config
     output['meta'] = meta
     output['image'] = image
@@ -214,17 +228,17 @@ def _sort_raytrace(input, max_lost=None):
 
     profiler.start('_sort_raytrace')
 
-    output = OrderedDict()
+    output = dict()
     output['config'] = input['config']
-    output['total'] = OrderedDict()
-    output['total']['meta'] = OrderedDict()
-    output['total']['image'] = OrderedDict()
-    output['found'] = OrderedDict()
-    output['found']['meta'] = OrderedDict()
-    output['found']['history'] = OrderedDict()
-    output['lost'] = OrderedDict()
-    output['lost']['meta'] = OrderedDict()
-    output['lost']['history'] = OrderedDict()
+    output['total'] = dict()
+    output['total']['meta'] = dict()
+    output['total']['image'] = dict()
+    output['found'] = dict()
+    output['found']['meta'] = dict()
+    output['found']['history'] = dict()
+    output['lost'] = dict()
+    output['lost']['meta'] = dict()
+    output['lost']['history'] = dict()
 
     output['total']['meta'] = input['meta']
     output['total']['image'] = input['image']
@@ -244,8 +258,8 @@ def _sort_raytrace(input, max_lost=None):
         w_lost = w_lost[index_lost[:max_lost]]
 
         for key_opt in key_opt_list:
-            output['found']['history'][key_opt] = OrderedDict()
-            output['lost']['history'][key_opt] = OrderedDict()
+            output['found']['history'][key_opt] = dict()
+            output['lost']['history'][key_opt] = dict()
 
             for key_ray in input['history'][key_opt]:
                 output['found']['history'][key_opt][key_ray] = input['history'][key_opt][key_ray][w_found]
@@ -255,23 +269,24 @@ def _sort_raytrace(input, max_lost=None):
 
     return output
 
+
 def combine_raytrace(input_list):
     """
     Produce a combined output from a list of raytrace outputs.
     """
     profiler.start('combine_raytrace')
 
-    output = OrderedDict()
+    output = dict()
     output['config'] = input_list[0]['config']
-    output['total'] = OrderedDict()
-    output['total']['meta'] = OrderedDict()
-    output['total']['image'] = OrderedDict()
-    output['found'] = OrderedDict()
-    output['found']['meta'] = OrderedDict()
-    output['found']['history'] = OrderedDict()
-    output['lost'] = OrderedDict()
-    output['lost']['meta'] = OrderedDict()
-    output['lost']['history'] = OrderedDict()
+    output['total'] = dict()
+    output['total']['meta'] = dict()
+    output['total']['image'] = dict()
+    output['found'] = dict()
+    output['found']['meta'] = dict()
+    output['found']['history'] = dict()
+    output['lost'] = dict()
+    output['lost']['meta'] = dict()
+    output['lost']['history'] = dict()
 
     num_iter = len(input_list)
     key_opt_list = list(input_list[0]['total']['meta'].keys())
@@ -279,7 +294,7 @@ def combine_raytrace(input_list):
 
     # Combine the meta data.
     for key_opt in key_opt_list:
-        output['total']['meta'][key_opt] = OrderedDict()
+        output['total']['meta'][key_opt] = dict()
         key_meta_list = list(input_list[0]['total']['meta'][key_opt].keys())
         for key_meta in key_meta_list:
             output['total']['meta'][key_opt][key_meta] = 0
@@ -333,6 +348,7 @@ def combine_raytrace(input_list):
     profiler.stop('combine_raytrace')
     return output
 
+
 def check_config(config):
     """
     Check the general section of the configuration dictionary.
@@ -350,6 +366,7 @@ def check_config(config):
             if not config['general']['make_directories']:
                 raise Exception('Output directory does not exist. Create directory or set make_directories to True.')
 
+
 def print_raytrace(results):
     """
     Print out some information and statistics from the raytracing results.
@@ -368,33 +385,3 @@ def print_raytrace(results):
         num_detector / num_source * 100))
     print('')
 
-def save_images(results):
-    """
-    Save images from the raytracing results.
-    """
-
-    rotate = False
-
-    prefix = results['config']['general']['output_prefix']
-    suffix = results['config']['general']['output_suffix']
-    run_suffix = results['config']['general']['output_run_suffix']
-    ext = results['config']['general']['image_extension']
-
-    if results['config']['general']['make_directories']:
-        os.makedirs(results['config']['general']['output_path'], exist_ok=True)
-
-    for key_opt in results['config']['optics']:
-        if key_opt in results['total']['image']:
-            if results['total']['image'][key_opt] is not None:
-                filename = '_'.join(filter(None, (prefix, key_opt, suffix, run_suffix)))+ext
-                filepath = os.path.join(results['config']['general']['output_path'], filename)
-
-                image_temp = results['total']['image'][key_opt]
-                if rotate:
-                    image_temp = np.rot90(image_temp)
-
-                generated_image = Image.fromarray(image_temp)
-                generated_image.save(filepath)
-
-                logging.info('Saved image: {}'.format(filepath))
-        
